#!/usr/bin/env python
# emusic_rmp_helper.py copyright (c) 2002 by Charl P. Botha http://cpbotha.net/
# $Id: emusic_rmp_helper.py,v 1.3 2002/12/09 08:45:11 cpbotha Exp $

# this code is under the GPL version 2 or later

# notes:
# - only one SERVER clause per RMP is supported
# - it's definitely not a full emp/rmp implementation

# QUICKSTART:
# 1. put the script somewhere convenient, make sure it's executable
#    (chmod +x emusic_rmp_helper.py)
# 2. modify dman_cmd and dman_params in the ### block below to suitable values
# 3. add the full pathname to emusic_rmp_helper.py as helper application for
#    application/vnd.rn-rn_music_package (extension rmp) in your browser
#    NB: it seems emusic.com has changed the mime-type, so add this script
#    as helper application for application/vnd.emusic-emusic_package
#    (extension emp) AS WELL (thanks to Tarragon Allen for this info)
# 4. buy a subscription and try downloading an album from emusic!

import getopt
import os
import re
import sys
import urlparse

#############################################################################
# DEFAULT VALUES, user can modify these to suit their config
#############################################################################
# set this to one if you just want to perform the actual downloads
test = 0
# this is the full path to your download manager
dman_cmd = '/usr/bin/nt'
# these are parameters you would like your download manager to be invoked with
# with d4x (nt) one can set the download path here, e.g.:
# dman_params = '-d /home/cpbotha/archive2/my-music/incoming'
dman_params = '' 
#############################################################################

def _disp_welcome():
    print("\n%s copyright (c) 2002 by Charl P. Botha http://cpbotha.net/\n" %
          os.path.basename(sys.argv[0]))

def _disp_usage():
    sys.stdout.write("%s [options] file1.rmp ... fileN.rmp\n" %
                     sys.argv[0])
    
    sys.stdout.write("""Options are:
    --help OR -h        : display this message
    --test OR -t        : do not actually download, just output URLs
    --dman-cmd OR -d    : specify full path of download manager to use
                          (default /usr/bin/nt i.e. \"d4x\")
    --dman-params OR -p : specify paramers that should be passed to the
                          download manager (default NONE)""")

    sys.stdout.write("""\n\nTo use this script in Mozilla:
    1. Go to Edit|Preferences|Navigator|Helper Applications
    2. Add/Edit MIME type application/vnd.rn-rn_music_package with extension
       \"rmp\".
    3. In the \"Application\" field, enter the full path of this script e.g.:
       /usr/local/bin/emusic_rmp_helper.py
       Also make sure that the script is executable.  Unfortunately, it seems
       that one can't add parameters to the helper script invocation, which
       means that if you want to have the download manager invoked with extra
       params, you'll have to modify the default variables above.""")

    sys.stdout.write("\n\nYou have to configure the download manager that "
                     "you're going to use with the\ncorrect download directory"
                     " for your MP3s.  You can do this in the GUI of the\n"
                     "download manager or by sending it certain parameters. "
                     "See the documentation\nabove for more details.\n\n")

def _make_track_url(netname, location, track_id, track_name):
    # make an initial url (by making use of the brilliant urlparse module)
    url_str = urlparse.urlunparse(('http', netname, location, '', '', ''))
    # now replace the %fid and %f with track_id and track_name
    url_str = url_str.replace('%fid', track_id).replace('%f', track_name)
    # if we're not in test mode, invoke our URL getter

    return url_str

def _get_tracks(urls, dman_cmd, dman_params, test):
    if test:
        print('In TEST mode - WOULD have invoked \"%s %s\" on:\n' %
              (dman_cmd, dman_params))
        for url in urls:
            print url

    else:
        cline = [dman_cmd] + dman_params.split() + urls 
        pid = os.spawnv(os.P_NOWAIT, dman_cmd, cline)
        sys.stderr.write("%d spawned\n" % pid)        


def _handle_rmp(rmpname, dman_cmd, dman_params, test):
    try:
        rmpfo = open(rmpname, 'r')
    except IOError:
        sys.stderr.write('ERROR: could not open %s.\n', rmpname)
        return 0

    rmp = rmpfo.read()

    # get server hostname and location on server
    ########################################################################

    # re.DOTALL means to match even EOL
    # we do non-greedy (minimal matching)
    sl_re = re.compile("<SERVER>.*?<NETNAME>(.*?)</NETNAME>.*?" \
                       "<LOCATION>(.*?)</LOCATION>.*?</SERVER>", re.DOTALL)
    sl_mo = sl_re.search(rmp)

    if sl_mo:
        netname = sl_mo.group(1)
        location = sl_mo.group(2)
    else:
        sys.stderr.write('ERROR: could not get netname/location from RMP.\n')
        return 0

    # now iterate through ALL the tracks and try to download them
    ########################################################################

    # re.DOTALL means to match even EOL
    # we do non-greedy (minimal matching)
    tid_re = re.compile("<TRACK>.*?<TRACKID>(.*?)</TRACKID>.*?" \
                        "<FILENAME>(.*?)</FILENAME>.*?</TRACK>", re.DOTALL)
    # findall returns a list of tuples of found groups
    track_groups = tid_re.findall(rmp)

    # assemble all urls
    urls = []
    for track in track_groups:
        urls.append(_make_track_url(netname, location, track[0], track[1]))

    # get the download manager going
    _get_tracks(urls, dman_cmd, dman_params, test)

def main():
    #b = open('/tmp/erw.log', 'w')
    #sys.stderr = b
    #sys.stdout = b
    
    _disp_welcome()

    global dman_cmd, dman_params, test
    
    try:
        # 'p:' means -p with something after
        optlist, args = getopt.getopt(
            sys.argv[1:], 'd:p:th',
            ['dman-cmd=', 'dman-params=', 'test', 'help'])
    except getopt.GetoptError,e:
        _disp_usage()
        sys.exit(1)

    for o, a in optlist:
        if o in ('-h', '--help'):
            _disp_usage()
            sys.exit(0)

        elif o in ('-d', '--dman-cmd'):
            dman_cmd = a

        elif o in ('-p', '--dman-params'):
            dman_params = a
            
        elif o in ('-t', '--test'):
            test = 1

    if len(args) == 0:
        _disp_usage()
        sys.exit(1)

    for rmpname in args:
        _handle_rmp(rmpname, dman_cmd, dman_params, test)

if __name__ == '__main__':
    main()
